#    Diagnostic and test task for the  Lyrtech SM (Signal Master)

set frame  SM
set title "Lyrtech SM Client"
set icon   SM

set started 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set LogFlag 1

set verbose 0

set SMServerList {}
set WebServer "select a server"

set SMResourceList {}
set SMResource ""

variable fpgadirectory
variable fpga1ffile
variable fpga2ffile

variable dspdirectory
variable dsp1ffile
variable dsp2ffile
variable dsp3ffile
variable dsp4ffile

set dspffmt 3

set dspramaddr  0
set dspramwords 16
set dspraminc 0

set dsprambase 0x90000000
set dsprambasechoice [list \
     "DSP Internal RAM - 0x00000000"  \
     "DSP External Memory interface CE0 - 0x80000000"  \
     "DSP External Memory interface CE1 - 0x90000000"  \
     "DSP External Memory interface CE2 - 0xA0000000"  \
     "DSP External Memory interface CE3 - 0xB0000000"  \
]
set dsprambaseindex 2

set boardID -1
set resourceID -1
set maxboards -1
set boardlist "none"



proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}


proc make-frame {} {
   global env

set rows 12
set cols 60

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :text WEBSERVERT         :label "SM WebServer" :w 20  :value "select a server"
    :choice-stack WEBSERVERC :label "" :strings ""
    :hskip 200
    :checkbox VERBOSE        :label "Enable Verbose Option" 

    :next-row T
    :number MODULE-NUM :label "SM Board Selection:" :min -1 :max -1 :w 5 :value -1
    :menubutton MODULE-SEL :label "" :menu "none"
    :hskip 100
    :button INFO :label "Report Board Information"  :command click-boardinfo
    :hskip 50
    :button OPTIONS :label "Reload Options" :command click-options

  :panel P1 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20; :row-gap 10

  :panel P3 :layout V :padx 20 :pady 5
    :layout H; :col-gap 20; :row-gap 10

    :button P51      :label "FPGA/DSP Programming" :w 20 :command "do_programming"
    :button CLEAR    :label "Clear log window"     :w 20 :command ":modify LOG :empty T"
    :next-row T

    :message REPORT  :label "Show FPGA registers" :w 20 :bold T
    :button REPORT1  :label "1" :w 2 :command "click-fpgaregisters 1"
    :button REPORT2  :label "2" :w 2 :command "click-fpgaregisters 2"

    :next-row T
    :message DISPLAY :label "Show DSP Memory" :w 20 :bold T
    :button DISPLAY1 :label "1" :w 2 :command "do_dspRAM 1"
    :button DISPLAY2 :label "2" :w 2 :command "do_dspRAM 2"
    :button DISPLAY3 :label "3" :w 2 :command "do_dspRAM 3"
    :button DISPLAY4 :label "4" :w 2 :command "do_dspRAM 4"

   open-frame

  :galley_size LOG $rows $cols

   :show T
}

proc do_programming {} {
    global fpga1ffile fpga2ffile
    global dsp1ffile dsp2ffile dsp3ffile dsp4ffile
    global dspffmt

  :frame "[self frame-name].pgm" :label "FPGA & DSP programming" :resize F :transient T

   :panel F1 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   FPGA1FILE    :label "Bitstream file for FPGA #1" :w 60 :value $fpga1ffile
      :button FPGA1BROWSE  :label "Browse"
      :next-row T
      :col-gap 20
      :button FPGA1UPLOAD    :label "Upload FPGA #1 bitstream file"  :command "do_UpLoadfpga 1"
      :button FPGA1PGM       :label "Program FPGA #1"                :command "do_Programfpga 1"
      :button FPGA1UPLOADPGM :label "Upload & Program FPGA #1"       :command "do_UpLoadProgramfpga 1"

   :panel F2 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   FPGA2FILE    :label "Bitstream file for FPGA #2" :w 60 :value $fpga2ffile
      :button FPGA2BROWSE  :label "Browse"
      :next-row T
      :col-gap 20
      :button FPGA2UPLOAD    :label "Upload FPGA #2 bitstream file"  :command "do_UpLoadfpga 2"
      :button FPGA2PGM       :label "Program FPGA #2"                :command "do_Programfpga 2"
      :button FPGA2UPLOADPGM :label "Upload & Program FPGA #2"       :command "do_UpLoadProgramfpga 2"

   :panel D :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack DSPFMT :label "File format for DSP" :valuex 20c :value $dspffmt \
          :strings [list "undefined" "Motorola S records" "Analog Devices Host Port Loadable" "binary"]

   :panel D1 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP1FILE      :label "Program file for DSP #1" :valuex 20c :w 60 :value $dsp1ffile
      :button DSP1BROWSE    :label "Browse"
      :next-row T
      :col-gap 20
      :button DSP1UPLOAD    :label "Upload DSP #1 program file"    :command "do_UpLoaddsp 1"
      :button DSP1PGM       :label "Program DSP #1"                :command "do_Programdsp 1"
      :button DSP1UPLOADPGM :label "Upload & Program DSP #1"       :command "do_UpLoadProgramdsp 1"

   :panel D2 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP2FILE      :label "Program file for DSP #2" :valuex 20c :w 60 :value $dsp2ffile
      :button DSP2BROWSE    :label "Browse"
      :next-row T
      :col-gap 20
      :button DSP2UPLOAD    :label "Upload DSP #2 program file"    :command "do_UpLoaddsp 2"
      :button DSP2PGM       :label "Program DSP #2"                :command "do_Programdsp 2"
      :button DSP2UPLOADPGM :label "Upload & Program DSP #2"       :command "do_UpLoadProgramdsp 2"

   :panel D3 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP3FILE      :label "Program file for DSP #3" :valuex 20c :w 60 :value $dsp3ffile
      :button DSP3BROWSE    :label "Browse"
      :next-row T
      :col-gap 20
      :button DSP3UPLOAD    :label "Upload DSP #3 program file"    :command "do_UpLoaddsp 3"
      :button DSP3PGM       :label "Program DSP #3"                :command "do_Programdsp 3"
      :button DSP3UPLOADPGM :label "Upload & Program DSP #3"       :command "do_UpLoadProgramdsp 3"

   :panel D4 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP4FILE      :label "Program file for DSP #4" :valuex 20c :w 60 :value $dsp4ffile
      :button DSP4BROWSE    :label "Browse"
      :next-row T
      :col-gap 20
      :button DSP4UPLOAD    :label "Upload DSP #4 program file"    :command "do_UpLoaddsp 4"
      :button DSP4PGM       :label "Program DSP #4"                :command "do_Programdsp 4"
      :button DSP4UPLOADPGM :label "Upload & Program DSP #4"       :command "do_UpLoadProgramdsp 4"


   :panel P0 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :button DONE      :label "Done" :command "frame-quit [self frame-name].pgm"

  :show T
}

proc do_dspRAM {id} {
   global dsprambase dspramaddr dspramwords dsprambasechoice dspraminc dsprambaseindex
   global SMResourceList resourceID

  :frame [self frame-name].dspram :label "Display DSP$id Memory" :resize F :transient T

    switch $id {
        1  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP0-PCI]}
        2  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP1-PCI]}
        3  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP2-PCI]}
        4  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP3-PCI]}
        default {midas-warning "Bad id ($id) for DSP access"; return}
    }	

    :panel DSPRAM :layout T :padx 10 :pady 10
     :layout H; :col-gap 10; :row-gap 10
     :choice-stack DSPRAMBASEC  :label "Area base address" :strings $dsprambasechoice :value $dsprambaseindex :valuex 24c
     :text DSPRAMBASET :label "" :value $dsprambase :w 12
     :next-row T
     :text DSPRAMADDR      :label "Address Offset"  :value $dspramaddr   :valuex 24c  :w 12
     :button DSPRAMADDRZ   :label "Reset to zero"
     :hskip 30
     :checkbox DSPRAMADDRI :label "Enable Address Increment" :value $dspraminc
     :next-row T
     :text DSPRAMWORDS   :label "Words to read"   :value $dspramwords  :valuex 24c  :w 12
     :hskip 25
     :text DSPRAMDATA    :label "Data to write"   :w 50
     :next-row T
     :layout V
     :message DSPRAMM1 :label "Memory Contents" :bold T
     :galley DSPRAMINFO :rows 20 :cols 110 :deaf T :font fixed

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

       :button DSPRAMDONE  :label "Done" :command "frame-quit [self frame-name].dspram"
       :button DSPRAMCLR   :label "Clear Window" :command ":modify DSPRAMINFO :append F"
       :button DSPRAMREAD  :label "Read Memory"  :command  do_dspramdisplay
       :button DSPRAMWRITE :label "Write Memory" :command  do_dspramwrite
  
   :show T
}

proc click-fpga0file {args} {click-fpgafile 0 [first $args]}
proc click-fpga1file {args} {click-fpgafile 1 [first $args]}
proc click-fpgafile {w f} {
    upvar #0 fpgadirectory  directory
    upvar #0 fpga[set w]ffile ffilename

    set ffilename $f
    set directory [file dirname $ffilename]

    Save_Options
}

proc click-fpga1browse {args} {click-fpgabrowse 1}
proc click-fpga2browse {args} {click-fpgabrowse 2}
proc click-fpgabrowse {w} {
    global env
    upvar #0 fpgadirectory  directory
    upvar #0 fpga[set w]ffile ffilename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.bit} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set ffilename $f   ;#  remember current choice
      set directory [file dirname $f]

    Save_Options

      :modify FPGA[set w]FILE :value $ffilename
}

proc do_Copy {file} {
   global env tcl_platform

   if {![info exists env(MIDASTempDir)] || ![file exists $env(MIDASTempDir)]} then {
       switch $tcl_platform(platform) {
          windows {set env(MIDASTempDir) [file join c:/ temp]}
          default {set env(MIDASTempDir) \tmp}
       }
   }

   catch {file mkdir $env(MIDASTempDir}
   catch {file mkdir [file join $env(MIDASTempDir) SMClient]}
   catch {file mkdir [file join $env(MIDASTempDir) SMServer]}

   set destn [file join $env(MIDASTempDir) SMClient [file tail $file]]
   set z [catch {file copy -force $file $destn} f]
   if {$z != 0} then {midas-warning "Cannot copy file $file to $destn\n$f"; return -1}

   set destn [file join $env(MIDASTempDir) SMServer [file tail $file]]
   set z [catch {file copy -force $file $destn} f]
   if {$z != 0} then {midas-warning "Cannot copy file $file to $destn\n$f"; return -1}

   return 0
}

proc do_UpLoadfpga {id} {
    upvar #0 fpga[set id]ffile file
    global WebServer

    if {$WebServer == "localhost"} {
        set rc [do_Copy $file]
        if {$rc == 0} {set-footer "[self frame-name].pgm"   "Uploaded file $file successful"}
    } else {
        set-footer "[self frame-name].pgm"  "Uploading FPGA bitstream file.... This takes about 10 secs"
        set z [catch {SMClient__SMUpLoad $file} rc]
        clear-footer "[self frame-name].pgm" 

        if {$z == 0} {
            set-footer "[self frame-name].pgm"   "Uploaded file $file successful"
        } else {
            bgerror "Uploading file $file failed\n$rc"
        }
    }
}

proc do_Programfpga {id} {        ;# id is 1 or 2
    global boardID SMResourceList
    upvar #0 fpga[set id]ffile file

    switch $id {
        1  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP0-PCI]}
        2  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP2-PCI]}
        default {midas-warning "Bad id ($id) for FPGA Load"; return}
    }	
    set-footer "[self frame-name].pgm"  "loading FPGA.... This takes about 10 secs"
    set z [catch {SMClient__SMSetFPGALoad [expr ($resourceID << 8) + $boardID] $file} rc]
    clear-footer "[self frame-name].pgm" 

    if {$rc == "NONE"} {global MIDAS_SOAPErrorInfo; set rc $MIDAS_SOAPErrorInfo}

    if {$z == 0} {
        if {$rc == 0} then {
             set ID [lindex $SMResourceList $resourceID]
             set-footer "[self frame-name].pgm"   "FpgaLoad for board $boardID $ID file $file successful"
        } else {
             midas-warning "FpgaLoad for board $boardID file $file failed\n$rc"
        }
    } else {
         midas-warning "FpgaLoad for board $boardID file $file failed\n$rc"
    }
}

proc do_UpLoadProgramfpga {id}  {do_UpLoadfpga $id; do_Programfpga $id}

proc click-dsp1file {args} {click-dspfile 1 [first $args]}
proc click-dsp2file {args} {click-dspfile 2 [first $args]}
proc click-dsp3file {args} {click-dspfile 3 [first $args]}
proc click-dsp4file {args} {click-dspfile 4 [first $args]}
proc click-dspfile {w f} {
    upvar #0 dspdirectory  directory
    upvar #0 dsp[set w]ffile dfilename

    set dfilename $f
    set directory [file dirname $dfilename]

    Save_Options
}


proc click-dspfmt {vl} {
    upvar #0 dspffmt format

    set format $vl
}

proc click-dsp1browse {args} {click-dspbrowse 1}
proc click-dsp2browse {args} {click-dspbrowse 2}
proc click-dsp3browse {args} {click-dspbrowse 3}
proc click-dsp4browse {args} {click-dspbrowse 4}
proc click-dspbrowse {w} {
    global env
    upvar #0 dspdirectory  directory
    upvar #0 dsp[set w]ffile dfilename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.out} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set dfilename $f   ;#  remember current choice
      set directory [file dirname $f]

    Save_Options

      :modify DSP[set w]FILE :value $dfilename
}

proc do_UpLoaddsp {id} {
    upvar #0 dsp[set id]ffile file
    global WebServer

    if {$WebServer == "localhost"} {
        set rc [do_Copy $file]
        if {$rc == 0} {set-footer "[self frame-name].pgm"   "Uploaded file $file successful"}
    } else {
        set-footer "[self frame-name].pgm"  "Uploading DSP program file.... This takes about 10 secs"
        set z [catch {SMClient__SMUpLoad $file} rc]
        clear-footer "[self frame-name].pgm" 

        if {$z == 0} {
            set-footer "[self frame-name].pgm"   "Uploaded file $file successful"
        } else {
            bgerror "Uploading file $file failed\n$rc"
        }
    }
}

proc do_Programdsp {id} {          ;#    id is 1 => 4
    global boardID  SMResourceList
    upvar #0 dsp[set id]ffile file
    upvar #0 dspffmt format

    switch $id {
        1  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP0-PCI]}
        2  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP1-PCI]}
        3  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP2-PCI]}
        4  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP3-PCI]}
        default {midas-warning "Bad id ($id) for DSP Load"; return}
    }	
    set-footer "[self frame-name].pgm"  "programming DSP.... This takes a few secs"
    set z [catch {SMClient__SMSetDSPLoad [expr ($resourceID << 8) + $boardID] $resourceID $format $file} rc]
    clear-footer "[self frame-name].pgm" 

    if {$rc == "NONE"} {global MIDAS_SOAPErrorInfo; set rc $MIDAS_SOAPErrorInfo}

    if {$z == 0} {
        if {$rc == 0} then {
             set ID [lindex $SMResourceList $resourceID]
             set-footer "[self frame-name].pgm"   "DspLoad for board $boardID $ID file $file successful"
        } else {
             midas-warning "DspLoad for board $boardID file $file failed\n$rc"
        }
    } else {
         midas-warning "DspLoad for board $boardID file $file failed\n$rc"
    }
}

proc do_UpLoadProgramdsp {id}  {do_UpLoaddsp $id; do_Programdsp $id}

proc redisplay {} {
    catch {update_}
}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n; update idletasks}


proc start args {
   global started env Options SMServerList

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

#    load application options and establish default values for any not defined

   load_Options

   if {![info exists Options(WebServer)]}     {set Options(WebServer) "localhost"}

   if {![info exists Options(fpgadirectory)]}  {set Options(fpgadirectory) $env(HOME)}
   if {![info exists Options(fpga1ffile)]}     {set Options(fpga1ffile) "undefined"}
   if {![info exists Options(fpga2ffile)]}     {set Options(fpga2ffile) "undefined"}

   if {![info exists Options(dspdirectory)]}   {set Options(dspdirectory) $env(HOME)}
   if {![info exists Options(dsp1ffile)]}      {set Options(dsp1ffile) "undefined"}
   if {![info exists Options(dsp2ffile)]}      {set Options(dsp2ffile) "undefined"}
   if {![info exists Options(dsp3ffile)]}      {set Options(dsp3ffile) "undefined"}
   if {![info exists Options(dsp4ffile)]}      {set Options(dsp4ffile) "undefined"}


   update_Options

   foreach option [list WebServer fpgadirectory fpga1ffile fpga2ffile dspdirectory dsp1ffile dsp2ffile dsp3ffile dsp4ffile] {
       global $option
       set $option $Options($option)
   }

   set SMServerList [list $WebServer]    ;# put default into the menu
   :modify WEBSERVERC :strings $SMServerList :value [lsearch $SMServerList $WebServer]

}

proc Save_Options {} {
   global Options

   foreach option [list WebServer fpgadirectory fpga1ffile fpga2ffile dspdirectory dsp1ffile dsp2ffile dsp3ffile dsp4ffile] {
       global $option
       set Options($option) [set $option]
   }

   update_Options
}

proc click-options {} {
   global Options

   load_Options

   foreach option [list WebServer fpgadirectory fpga1ffile fpga2ffile dspdirectory dsp1ffile dsp2ffile dsp3ffile dsp4ffile] {
       global $option
       set $option $Options($option)
   }
}

proc click arg {
   global verbose WebServer

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer

   switch -glob $w  {

      WEBSERVERT    {click-server $v}
      WEBSERVERC    {click-server $i}

      MODULE-SEL    -
      MODULE-NUM    {click-select-module $v}

      VERBOSE       {set verbose $v; if {$WebServer != ""} then {SMClient__SMVerbose $verbose}}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }

    redisplay 
}

proc click-server {v} {
    global env SMServerList WebServer
    global SMResourceList
    global boardID resourceID maxboards boardlist
    global verbose started

    global SMHost SMPort

    set WebServer $v
    set SMServerList [list $WebServer]    ;# put default into the menu
    :modify WEBSERVERT :value $WebServer
    :modify WEBSERVERC :strings $SMServerList :value [lsearch $SMServerList $WebServer]

    Save_Options

    set SMHost $WebServer
    loadsource SMClient     ;# better solution needed

    SMClient__SMVerbose $verbose
    set maxboards [SMClient__SMInit]
    
    set boardlist {}
    for {set i 0} {$i < $maxboards} {incr i} {lappend boardlist $i}
    set boardID 0
    :modify MODULE-SEL :menu $boardlist
    :modify MODULE-NUM :min 0 :max [expr $maxboards-1]
    :modify MODULE-NUM :value $boardID

    insert "Initialised SM Web Service at $WebServer: $maxboards SM boards discovered"

    set started 2

    set SMResourceList [split [SMClient__SMGetUsers $boardID] |]

    insert "Discovered resources: $SMResourceList"
}

proc click-select-module {vl} {
    global boardID maxboards

    set boardID $vl
    if {$boardID < 0} {set boardID 0}
    if {$boardID >= $maxboards} {set boardID [expr $maxboards-1]}

    :modify MODULE-NUM :value $boardID
}

proc update_ {} {
    global boardID

}

proc click-boardinfo {args} {
    global boardID

   set type [SMClient__SMGetSMType $boardID]
   switch $type {
       0x403   {insert "Board Type is QuadC6xx-based SignalMaster"}
       default {insert "Unexpected Board Type ($type)"}
   }
   
   foreach ID {1 2 0x100 0x200 0x201 0x400 0x401} {
       set version [SMClient__SMGetVersion $boardID $ID]
       set desc [lreplace $version 0 2]
       if {$desc != "n/a"} {insert "$desc version [expr [first $version]].[expr [second $version]].[expr [third $version]]"}
   }
}

proc click-fpgaregisters {id} {
   global SMResourceList boardID
   global DataRAM

    switch $id {
        1  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP0-PCI]}
        2  {set resourceID [lsearch $SMResourceList SMQUADC6XX-DSP2-PCI]}
        default {midas-warning "Bad id ($id) for FPGA"; return}
    }	

   set-footer "Reading data from DSP RAM"

   set baseaddress 0x90000000
   set length 1
   set DataRAM [SMClient__SMGetDSPRead [expr ($resourceID << 8) + $boardID] $baseaddress $length]

   insert "FPGA #$id version [format 0x%08x [first $DataRAM]]"

   set baseaddress 0x90000040
   set length 16
   set DataRAM [SMClient__SMGetDSPRead [expr ($resourceID << 8) + $boardID] $baseaddress $length]

   set info ""
   foreach word $DataRAM {
       append info "[format 0x%08x $word]  "
   }
   
   insert "User control registers: $info"

   clear-footer
}

proc  do_dspramdisplay {} {
   global boardID resourceID
   global  dsprambase dspramaddr dspramwords dspraminc
   global DataRAM

#   if {$resourceID > 3} {midas-warning "You cannot only do this for a DSP resource"; return}

   set-footer [self frame-name].dspram "Reading data from DSP RAM"

   set baseaddress [expr $dsprambase + $dspramaddr]
   set length [expr $dspramwords]
   set DataRAM [SMClient__SMGetDSPRead [expr ($resourceID << 8) + $boardID] $baseaddress $length]

   set info ""
   set i 0
   set offset 0
   foreach word $DataRAM {
      if {$i == 0} {append info "[format "0x%08x" [expr $baseaddress + $offset]]: "}
      append info "[format 0x%08x $word]  "
      incr i
      if {$i == 8} then {append info "\n"; set i 0}
      incr offset 4
   }
   :modify DSPRAMINFO :strings $info\n

   clear-footer [self frame-name].dspram

   if {$dspraminc} {click-dspramaddr [expr $dspramaddr + ($dspramwords*4)]}
}

proc click-dsprambasec {v} {
   global  dsprambase dsprambasechoice

   set dsprambase  [last [lindex $dsprambasechoice $v]]
   :modify DSPRAMBASET :value [format "0x%08x" $dsprambase]
}

proc click-dsprambaset {v} {
   global  dsprambase
   if {[scan-int $v] == "???"} then {return}
   set dsprambase $v
   :modify DSPRAMBASET :value [format "0x%08x" $dsprambase]
   :modify DSPRAMBASEC :value -1
}

proc click-dspramaddr {v} {
   global dspramaddr
   if {[scan-int $v] == "???"} then {return}
   set dspramaddr $v
   :modify DSPRAMADDR :value [format "0x%08x" $dspramaddr]
}

proc click-dspramaddrz {args} {click-dspramaddr 0}

proc click-dspramwords {v} {
   global dspramwords
   if {[scan-int $v] == $v && $v >= 0 && $v <= 0x400} then {set dspramwords $v}
   :modify DSPRAMWORDS :value [format "0x%02x" $dspramwords]
}

proc click-dspramaddri {v} {
    global dspraminc
    set dspraminc $v
}

proc click-dspramdata {v} {
    global dspramdata
    set dspramdata $v
}

proc do_dspramwrite {} {
    global boardID resourceID
    global  dsprambase dspramaddr dspramwords dspraminc dspramdata
    global DataRAM

#   if {$resourceID > 3} {midas-warning "You cannot only do this for a DSP resource"; return}

    set baseaddress [expr $dsprambase + $dspramaddr]
    set length [llength $dspramdata]
    set DataRAM $dspramdata

    SMClient__SMSetDSPWrite [expr ($resourceID << 8) + $boardID] $baseaddress $length $DataRAM

}


#    The Options are experiment specific and normally used by SYS
#    However the same code as for the session preferences is used which allows for wider use in the future.

proc get_Option_file {} {
    global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

proc put_Option_file {lines} {
   global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf w]} m]
    if {$z != 0} then {
        insert-log "Unable to access $pf - $m"
        return 1
    }

    foreach item $lines {
      if {$item != ""} {puts $f $item}
    }

    close $f
    return 0
}

#    save all Options for the current task
proc update_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
   }

   lappend lines "## $key"
   foreach item [array names Options] {
       lappend lines "$item $Options($item)"
   }
   lappend lines "##"

   set z [put_Option_file $lines]
   return $z
}

#    load all Options for the current task
proc load_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           set Options([head $item]) [tail $item]
       }
   }
}

#    return task Options
proc get_Options {task} {
   global env

   set lines [get_Option_file]
   set key $task
   set range [find-keyword $lines $key]

   set Options {}

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           lappend Options [list [first $item] [second $item]]
       }
   } else {
       lappend Options [list Options.filename  "[file join $env(HOME) MidasOptions Options]"]
   }
   return $Options
}
