#    Control module for the TDR Interface Card (TDRI)

set frame  TDRI
set title "TDR Interface Card"
set icon   TDRI

set started 0

set WebServer "localhost"

proc bgerror args {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n[lindex $args 0]" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}

proc click_redisplay {args} {Redisplay}

proc click_dac1sel {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xfc) + [first $i]]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_dac2sel {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xf3) + ([first $i] << 2)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_veto {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xef) + ([first $i] << 4)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_fp {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xdf) + ([first $i] << 5)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc Redisplay {} {

    set rc [catch {TDRIAccessClient__Version} m]
    if {$rc != 0} {bgerror; return}
    :modify VERS :value $m

    set rc [catch {TDRIAccessClient__Status} m]
    if {$rc != 0} {bgerror; return}

    :modify ST0 :value [expr $m & 1]
    :modify VETO :value [expr ($m >> 1) & 1]

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}
    :modify DAC1SEL :value [expr $m & 3]
    :modify DAC2SEL :value [expr ($m >> 2) & 3]
    :modify VETO    :value [expr ($m >> 4) & 1]
    :modify FP      :value [expr ($m >> 5) & 1]

    set-footer "Control = [format "0x%x" $m]"

}

proc click arg {
    global actonall

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p [list $v $i]} else {insert-debug "$arg"}
      }
   }
}

proc start args {
   global started
   global WebServer TDRIAccessServerHost

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

   set TDRIAccessServerHost $WebServer
   loadsource TDRIAccessClient     ;# better solution needed
   package require TDRIAccessClient

   TDRIAccessClient__Init

   Redisplay
}

proc make-frame {} {
   global env

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P1 :layout T :padx 5 :pady 5
    :layout V; :col-gap 10; :row-gap 10

    :choice DAC1SEL :label "Select input for DAC1 visualisation" :strings [list 1 2 3 4]  :valuex 35c
    :choice DAC2SEL :label "Select input for DAC2 visualisation" :strings [list 1 2 3 4]  :valuex 35c
    :choice VETO    :label "Veto mode select"    :strings [list loopback normal] :valuex 35c
    :choice FP      :label "TDR Clk Sync select" :strings [list off on] :valuex 35c

  :panel P2 :layout T :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :checkbox ST0   :label "High Temperature" :deaf T
    :text VERS      :label "firmware version" :w 6 :deaf T

  :panel BUTTONS :after footer :layout B :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

     :button REDISPLAY

   :show T
}

